@@ -4,6 +4,9 @@ import colorama
 import os
 import json
 import time
+import urllib.parse
+from pathlib import Path
+
 from .DataHandler import (
     game_paths,
     load_json_file,
@@ -78,7 +81,7 @@ class MegaMixContext(SuperContext):
         super().__init__(server_address, password)
         self.ready_callback = ready_callback
         self.error_callback = error_callback
-        self.username = slot_name
+        self.username = urllib.parse.urlparse(server_address).username
         self.game = "Hatsune Miku Project Diva Mega Mix+"
         self.path = game_paths().get("mods")
         self.mod_name = "ArchipelagoMod"
@@ -86,6 +89,9 @@ class MegaMixContext(SuperContext):
         self.songResultsLocation = f"{self.path}/{self.mod_name}/results.json"
         self.deathLinkInLocation = f"{self.path}/{self.mod_name}/death_link_in"
         self.deathLinkOutLocation = f"{self.path}/{self.mod_name}/death_link_out"
+        self.trapSuddenLocation = f"{self.path}/{self.mod_name}/sudden"
+        self.trapHiddenLocation = f"{self.path}/{self.mod_name}/hidden"
+        self.trapIconLocation = f"{self.path}/{self.mod_name}/icontrap"
         self.modData = None
         self.modded = False
         self.freeplay = False
@@ -224,8 +230,17 @@ class MegaMixContext(SuperContext):
                     elif network_item.item == 2:
                         # Maybe move static items out of MegaMixCollection instead of hard coding?
                         pass
+                    elif network_item.item == 4:
+                        if not os.path.isfile(self.trapHiddenLocation):
+                            Path(self.trapHiddenLocation).touch()
+                    elif network_item.item == 5:
+                        if not os.path.isfile(self.trapSuddenLocation):
+                            Path(self.trapSuddenLocation).touch()
+                    elif network_item.item == 9:
+                        if not os.path.isfile(self.trapIconLocation):
+                            Path(self.trapIconLocation).touch()
                     else:
-                        ids_to_packs.setdefault(self.song_id_to_pack(network_item.item), []).append(network_item.item)
+                        ids_to_packs.setdefault(self.song_id_to_pack(network_item.item), set()).add(network_item.item)
 
             for song_pack in ids_to_packs:
                 song_unlock(self.path, ids_to_packs.get(song_pack), False, song_pack)
@@ -244,7 +259,7 @@ class MegaMixContext(SuperContext):
                 logger.info(f"Got enough leeks! Unlocking goal song: {self.goal_song}")
 
             song_pack = self.song_id_to_pack(self.goal_id)
-            song_unlock(self.path, [self.goal_id], False, song_pack)
+            song_unlock(self.path, {self.goal_id}, False, song_pack)
 
 
     async def watch_json_file(self, file_name: str):
@@ -297,7 +312,6 @@ class MegaMixContext(SuperContext):
 
     def on_deathlink(self, data: dict[str, any]):
         super().on_deathlink(data)
-        from pathlib import Path
         Path(self.deathLinkInLocation).touch()
 
 
@@ -422,16 +436,16 @@ class MegaMixContext(SuperContext):
     async def freeplay_toggle(self):
         self.freeplay = not self.freeplay
 
-        song_ids = [location_id for location_id in list(self.location_ids)[::self.checks_per_song]
-                    if location_id not in [i.item for i in self.previous_received]]
+        song_ids = {location_id for location_id in sorted(self.location_ids)[::self.checks_per_song]
+                    if location_id not in [i.item for i in self.previous_received]}
 
         if not self.freeplay:
-            song_ids = [received.item for received in self.previous_received if received.item in self.missing_checks]
+            song_ids = {received.item for received in self.previous_received if received.item in self.missing_checks}
 
             if self.leeks_obtained >= self.leeks_needed:
-                song_ids.append(self.goal_id)
+                song_ids.add(self.goal_id)
         elif self.leeks_obtained < self.leeks_needed:
-            song_ids.append(self.goal_id)
+            song_ids.add(self.goal_id)
 
         freeplay_song_list(self.mod_pv_list, song_ids, self.freeplay)
 
@@ -466,7 +480,7 @@ class MegaMixContext(SuperContext):
             self.watch_death_link_task = asyncio.create_task(self.watch_death_link_out(self.deathLinkOutLocation))
 
 
-def launch(server_address: str = None, slot_name: str = None, password: str = None, ready_callback=None, error_callback=None):
+def launch(server_address: str = None, password: str = None, ready_callback=None, error_callback=None):
     """
     Launch the client
     """
@@ -474,7 +488,7 @@ def launch(server_address: str = None, slot_name: str = None, password: str = No
     logging.getLogger("MegaMixClient")
 
     async def main():
-        ctx = MegaMixContext(server_address, slot_name, password, ready_callback, error_callback)
+        ctx = MegaMixContext(server_address, password, ready_callback, error_callback)
         if ctx._can_takeover_existing_gui():
             await ctx._takeover_existing_gui() 
         else:
@@ -509,6 +523,6 @@ def launch(server_address: str = None, slot_name: str = None, password: str = No
             error_callback()
 
 
-def main(server_address: str = None, slot_name: str = None, password: str = None, ready_callback=None, error_callback=None):
+def main(server_address: str = None, password: str = None, ready_callback=None, error_callback=None):
     """Main entry point for integration with MultiWorld system"""
-    launch(server_address, slot_name, password, ready_callback, error_callback)
+    launch(server_address, password, ready_callback, error_callback)
